#!/bin/sh

: ${LIVENESS_THRESHOLD_SECONDS:=600}
: ${AWS_REGION:=us-west-2}

function check_cloudwatch() {
  if [[ -f "/usr/bin/aws" && "$CLOUDWATCH_LOG_GROUP" != "" ]]; then
    echo "Running cloudwatch log group check."
    log_group=$CLOUDWATCH_LOG_GROUP
    if [[ "$log_group" == "" ]]; then
      aws logs describe-log-groups --region "$AWS_REGION"
    else
      aws logs describe-log-groups --region "$AWS_REGION" --log-group "$log_group"
    fi
    log_group_result=$?
    if [[ "$log_group_result" != "0" ]]; then
      echo "Log group check result code: $log_group_result"
      exit 1
    fi
  else
    echo "Skipping cloudwatch log group check."
  fi
}

function check_buffer_last_modified() {
  if [[ "$BUFFER_FOLDER" == "" ]]; then
    echo "Skipping buffer check."
  else
    echo "Running checks on buffer folder: $BUFFER_FOLDER"
    LAST_MODIFIED_DATE=`stat $BUFFER_FOLDER | grep Modify | sed -r "s/Modify: (.*)/\1/"`
    LAST_MODIFIED_TIMESTAMP=`date -d "$LAST_MODIFIED_DATE" +%s`
    echo "Last modified timestamp on buffer folder '$BUFFER_FOLDER': $LAST_MODIFIED_DATE"
    if [[ `date +%s` -gt `expr $LAST_MODIFIED_TIMESTAMP + $LIVENESS_THRESHOLD_SECONDS` ]]; then 
      echo "Last modified timestamp reached threshold ($LIVENESS_THRESHOLD_SECONDS seconds)"
      exit 1
    fi
  fi
}

function main() {
    while [[ $# -gt 0 ]]
    do
      key="$1"
      case $key in
        -c|--cloudwatch-log-group)
          CLOUDWATCH_LOG_GROUP="$2"
          shift 2
        ;;
        -b|--buffer-folder)
          BUFFER_FOLDER="$2"
          shift 2
        ;;
        -l|--liveness-threshold-seconds)
          LIVENESS_THRESHOLD_SECONDS="$2"
          shift 2
        ;;
        *)
          echo "Unknown option: $1"
          echo "Available options: -c|--cloudwatch-log-group, -b|--buffer-folder, -l|--liveness-threshold-seconds"
          exit 1
        ;;
      esac
    done
    check_cloudwatch
    check_buffer_last_modified
    echo "Checks successfully finished."
}

main ${1+"$@"}